import { Bucket } from "../../miniplex-bucket/dist/miniplex-bucket.cjs.js";
export * from "../../miniplex-bucket/dist/miniplex-bucket.cjs.js";
export type Predicate<E, D extends E> = ((v: E) => v is D) | ((entity: E) => boolean);
/**
 * A utility type that marks the specified properties as required.
 */
export type With<E, P extends keyof E> = E & Required<Pick<E, P>>;
export type Without<E, P extends keyof E> = Omit<E, P>;
/**
 * A utility type that removes all optional properties.
 */
export type Strict<T> = WithoutOptional<T>;
type OptionalKeys<T> = {
    [K in keyof T]-?: undefined extends T[K] ? K : never;
};
type WithoutOptional<T> = Pick<T, Exclude<keyof T, OptionalKeys<T>[keyof T]>>;
type QueryConfiguration = {
    with: any[];
    without: any[];
    predicates: Function[];
};
interface IQueryableBucket<E> {
    /**
     * Queries for entities that have all of the given components. If this is called on
     * an existing query, the query will be extended to include this new criterion.
     *
     * @param components The components to query for.
     */
    with<C extends keyof E>(...components: C[]): Query<With<E, C>>;
    /**
     * Queries for entities that have none of the given components. If this is called on
     * an existing query, the query will be extended to include this new criterion.
     *
     * @param components The components to query for.
     */
    without<C extends keyof E>(...components: C[]): Query<Without<E, C>>;
    /**
     * Queries for entities that match the given predicate. If this is called on
     * an existing query, the query will be extended to include this new criterion.
     *
     * @param predicate The predicate to query for.
     */
    where<D extends E>(predicate: Predicate<E, D>): Query<D>;
}
export declare class World<E extends {} = any> extends Bucket<E> implements IQueryableBucket<E> {
    constructor(entities?: E[]);
    update(entity: E): E;
    update<C extends keyof E>(entity: E, component: C, value: E[C]): E;
    update(entity: E, update: Partial<E>): E;
    update(entity: E, fun: (entity: E) => Partial<E> | void): E;
    addComponent<C extends keyof E>(entity: E, component: C, value: E[C]): void;
    removeComponent(entity: E, component: keyof E): void;
    protected queries: Set<Query<any>>;
    query<D>(config: QueryConfiguration): Query<D>;
    with<C extends keyof E>(...components: C[]): Query<With<E, C>>;
    without<C extends keyof E>(...components: C[]): Query<Without<E, C>>;
    where<D extends E>(predicate: Predicate<E, D>): Query<D>;
    protected reindex(entity: E, future?: E): void;
    private entityToId;
    private idToEntity;
    private nextId;
    id(entity: E): number | undefined;
    entity(id: number): E | undefined;
}
export declare class Query<E> extends Bucket<E> implements IQueryableBucket<E> {
    world: World;
    config: QueryConfiguration;
    protected _isConnected: boolean;
    /**
     * True if this query is connected to the world, and will automatically
     * re-evaluate when entities are added or removed.
     */
    get isConnected(): boolean;
    /**
     * A unique, string-based key for this query, based on its configuration.
     */
    readonly key: string;
    constructor(world: World, config: QueryConfiguration);
    get entities(): E[];
    [Symbol.iterator](): {
        next: () => {
            value: E;
            done: boolean;
        };
    };
    connect(): this;
    disconnect(): this;
    with<C extends keyof E>(...components: C[]): Query<With<E, C>>;
    without<C extends keyof E>(...components: C[]): Query<Without<E, C>>;
    where<D extends E>(predicate: Predicate<E, D>): Query<D>;
    want(entity: E): boolean;
    evaluate(entity: any, future?: any): void;
}
