import { With } from "./core.js";
/**
 * Checks if an entity has all the given components.
 *
 * @param entity The entity to check
 * @param components The components to check for
 * @returns True if all components are present
 */
export declare function hasComponents<E, C extends keyof E>(entity: E, ...components: C[]): entity is With<E, C>;
/**
 * Checks if an entity has any of the given components.
 *
 * @param entity The entity to check
 * @param components The components to check for
 * @returns True if any component is present
 */
export declare function hasAnyComponents<E>(entity: E, ...components: (keyof E)[]): boolean;
/**
 * Checks if an entity has none of the given components.
 *
 * @param entity The entity to check
 * @param components The components to check for
 * @returns True if no component is present
 */
export declare function hasNoComponents<E>(entity: E, ...components: (keyof E)[]): boolean;
